@@ -24,6 +24,8 @@
 
 #include "dynstring.h"
 
+struct _environment;
+
 #define NUMBER_INT       1
 #define NUMBER_DOUBLE    2
 #define NUMBER_STRING    3
@@ -44,4 +46,4 @@ typedef struct
 void arithScanFromDS (dynstring *ds);
 void arithEndScanning (void);
 
-dynstring arithEvalDS (dynstring *ds);
+dynstring arithEvalDS (dynstring *ds, struct _environment *env);
