@@ -22,15 +22,23 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+#include "../config.h"
+
 #include <assert.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
 #include <sys/stat.h>
+#include <sys/param.h>
 #include <unistd.h>
-#ifdef _NEXT
-#include <libc.h>
+
+#ifndef PATH_MAX
+#ifdef MAXPATHLEN
+#define PATH_MAX MAXPATHLEN
+#else
+#define PATH_MAX 1024
+#endif
 #endif
 
 #include "../memory.h"
@@ -100,7 +108,7 @@ builtInFopen (int numArgs, macroArgument *args, environment *env, outputWriter *
 	openMode = args[1].value.value->v.scalar.scalar.data;
     }
 
-    if (strcmp(openMode, "r") != 0 && strcmp(openMode, "w") != 0)
+    if (strcmp(openMode, "r") != 0 && strcmp(openMode, "w") != 0 && strcmp(openMode, "a") != 0 )
     {
 	OUT_STRING(ow, "-1", 2);
 	return;
@@ -125,8 +133,9 @@ builtInFpipe (int numArgs, macroArgument *args, environment *env, outputWriter *
     int thePipe[2];
     int fileNum;
     char fileNumString[64];
+    enum { PIPE_READ, PIPE_WRITE } pipeMode = PIPE_READ;
 
-    if (!(numArgs >= 1))
+    if (!(numArgs >= 2))
     {
 	issueError(ERRMAC_WRONG_NUM_ARGS, "fpipe");
 	return;
@@ -138,6 +147,21 @@ builtInFpipe (int numArgs, macroArgument *args, environment *env, outputWriter *
 	OUT_STRING(ow, "-1", 2);
     }
 
+    transformArgumentToScalar(&args[0]);
+    
+    if (args[0].value.value->v.scalar.scalar.length > 1 ||
+	(args[0].value.value->v.scalar.scalar.length == 1 && 
+	 strcmp(args[0].value.value->v.scalar.scalar.data, "r") != 0 &&
+	 strcmp(args[0].value.value->v.scalar.scalar.data, "w") != 0) )
+    {
+	issueError(ERRMAC_INVALID_MACRO_ARG, "fpipe", args[0].value.value->v.scalar.scalar.data, 0);
+	OUT_STRING(ow, "-1", 2);
+    }
+
+    if (args[0].value.value->v.scalar.scalar.length == 1 &&
+	strcmp(args[0].value.value->v.scalar.scalar.data, "w") == 0)
+        pipeMode = PIPE_WRITE;
+
     switch (fork())
     {
 	case -1 :
@@ -149,12 +173,21 @@ builtInFpipe (int numArgs, macroArgument *args, environment *env, outputWriter *
 		int i;
 		char **execArgs = (char**)memXAlloc((numArgs + 1) * sizeof(char*));
 
-		close(thePipe[0]);
-		dup2(thePipe[1], 1);
+		switch (pipeMode)
+		{
+		  case PIPE_READ:
+		    close(thePipe[0]);
+		    dup2(thePipe[1], 1);
+		    break;
 
-		for (i = 0; i < numArgs; ++i)
-		    execArgs[i] = transformArgumentToScalar(&args[i])->v.scalar.scalar.data;
-		execArgs[i] = 0;
+		  default:
+		    close(thePipe[1]);
+		    dup2(thePipe[0], 0);
+		}
+
+		for (i = 1; i < numArgs; ++i)
+		    execArgs[i-1] = transformArgumentToScalar(&args[i])->v.scalar.scalar.data;
+		execArgs[i-1] = 0;
 
 		if (execvp(execArgs[0], execArgs) == -1)
 		{
@@ -165,10 +198,19 @@ builtInFpipe (int numArgs, macroArgument *args, environment *env, outputWriter *
 	    break;
 
 	default :
-	    close(thePipe[1]);
-
 	    fileNum = getNewFileNum();
-	    fileTable[fileNum] = fdopen(thePipe[0], "r");
+
+	    switch (pipeMode)
+	    {
+	      case PIPE_READ:
+		close(thePipe[1]);
+		fileTable[fileNum] = fdopen(thePipe[0], "r");
+		break;
+
+	      default:
+		close(thePipe[0]);
+		fileTable[fileNum] = fdopen(thePipe[1], "w");
+	    }
 
 	    if (fileTable[fileNum] == 0)
 	    {
@@ -353,11 +395,15 @@ builtInFstat (int numArgs, macroArgument *args, environment *env, outputWriter *
 	sprintf(numberString, "%lu", (unsigned long)buf.st_size);
 	valueHashDefine(theHash, &sizeName, valueNewScalarFromCString(numberString));
 
+#ifdef HAVE_ST_BLKSIZE
 	sprintf(numberString, "%lu", (unsigned long)buf.st_blksize);
 	valueHashDefine(theHash, &blksizeName, valueNewScalarFromCString(numberString));
+#endif
 
+#ifdef HAVE_ST_BLOCKS
 	sprintf(numberString, "%lu", (unsigned long)buf.st_blocks);
 	valueHashDefine(theHash, &blocksName, valueNewScalarFromCString(numberString));
+#endif
 
 	sprintf(numberString, "%lu", (unsigned long)buf.st_atime);
 	valueHashDefine(theHash, &atimeName, valueNewScalarFromCString(numberString));
@@ -372,6 +418,33 @@ builtInFstat (int numArgs, macroArgument *args, environment *env, outputWriter *
     }
 }
 
+void
+builtInFgetwd (int numArgs, macroArgument *args, environment *env, outputWriter *ow)
+{
+    char buffer[PATH_MAX];
+
+    if (!(numArgs == 0))
+    {
+	issueError(ERRMAC_WRONG_NUM_ARGS, "fgetwd");
+	return;
+    }
+
+    getcwd(buffer, PATH_MAX);
+    OUT_STRING(ow, buffer, strlen(buffer));
+}
+
+void
+builtInFchdir (int numArgs, macroArgument *args, environment *env, outputWriter *ow)
+{
+    if (!(numArgs == 1))
+    {
+	issueError(ERRMAC_WRONG_NUM_ARGS, "fchdir");
+	return;
+    }
+
+    chdir(transformArgumentToScalar(&args[0])->v.scalar.scalar.data);
+}
+
 void
 registerFileOps (void)
 {
@@ -392,4 +465,6 @@ registerFileOps (void)
     registerBuiltIn("fputs", builtInFputs, 1, 0, 0);
     registerBuiltIn("feof", builtInFeof, 1, 0, 0);
     registerBuiltIn("fstat", builtInFstat, 1, 0, 0);
+    registerBuiltIn("fgetwd", builtInFgetwd, 1, 0, 0);
+    registerBuiltIn("fchdir", builtInFchdir, 1, 0, 0);
 }
