@@ -27,12 +27,18 @@
 
 struct _value;
 struct _outputWriter;
+struct _bcSubscript;
+struct _environment;
 
 extern char *metaChar;
 extern char quoteChar;
 
-typedef void (*internalSet) (struct _value *theValue);
-typedef void (*internalGet) (struct _outputWriter *ow);
+typedef void (*internalSet) (struct _value *theValue, int needCopy,
+			     struct _bcSubscript *subscripts,
+			     struct _environment *env);
+typedef void (*internalGet) (struct _bcSubscript *subscripts,
+			     struct _environment *env,
+			     struct _outputWriter *ow);
 
 void registerInternals (void);
 
